from typing import Any


class DfiberException(Exception):
    """Base exception for all Dfiber-related errors."""

    def __init__(self, error_type: Any, name: Any, detail: Any, module_code: int = 0x00):
        self.type = error_type
        self.name = name
        self.detail = detail
        self.module_code = module_code

    def __str__(self):
        return f"(Error Module Code: {self.module_code:02x}, Type: {self.type}, Name:{self.name}, Detail: {self.detail})"


class DfiberConnectionException(DfiberException):
    """Connection errors (e.g., failure to connect to Trino)."""
    module_code = 0x01

    def __init__(self, error_type: Any, name: Any, detail: Any):
        super().__init__(error_type, name, detail, self.module_code)


class DfiberValueError(DfiberException):
    """Value errors such as missing or invalid parameters."""
    module_code = 0x02

    def __init__(self, error_type: Any, name: Any, detail: Any):
        super().__init__(error_type, name, detail, self.module_code)


class DfiberQueryException(DfiberException):
    """Errors encountered while executing queries."""
    module_code = 0x03

    def __init__(self, error_type: Any, name: Any, detail: Any):
        super().__init__(error_type, name, detail, self.module_code)


class DfiberOperationException(DfiberException):
    """Errors arising during operations, both internal or external."""
    module_code = 0x04

    def __init__(self, error_type: Any, name: Any, detail: Any):
        super().__init__(error_type, name, detail, self.module_code)


class DfiberAuthenticationException(DfiberException):
    """Errors related to authentication failures."""
    module_code = 0x05

    def __init__(self, error_type: Any, name: Any, detail: Any):
        super().__init__(error_type, name, detail, self.module_code)


class DfiberTypeError(DfiberException):
    """Errors related to type mismatches."""
    module_code = 0x06

    def __init__(self, error_type: Any, name: Any, detail: Any):
        super().__init__(error_type, name, detail, self.module_code)
